@@ -4,25 +4,22 @@
 #' variables using several indicators, as shown by Olivoto et al. (2017).
 #'
 #'
-#' @param .data The data to be analyzed. Must be a symmetric correlation matrix
-#'   or, a dataframe containing the predictor variables, or an object of class
-#'   \code{split_factors}.
+#' @param .data The data to be analyzed. It must be a symmetric correlation
+#'   matrix, or a data frame, possible with grouped data passed from
+#'   \code{\link[dplyr]{group_by}()}.
 #' @param ... Variables to use in the correlation. If \code{...} is null then
 #'   all the numeric variables from \code{.data} are used. It must be a single
 #'   variable name or a comma-separated list of unquoted variables names.
-#' @param by One variable (factor) to split the data into subsets. The function
-#'   is then applied to each subset and returns a list where each element
-#'   contains the results for one level of the variable in \code{by}. To split
-#'   the data by more than one factor variable, use the function
-#'   \code{\link{split_factors}} to pass subsetted data to \code{.data}.
+#'@param by One variable (factor) to compute the function by. It is a shortcut
+#'  to \code{\link[dplyr]{group_by}()}. To compute the statistics by more than
+#'  one grouping variable use that function.
 #' @param n If a correlation matrix is provided, then \code{n} is the number of
 #'   objects used to compute the correlation coefficients.
-#' @param verbose If \code{verbose = TRUE} then some results are shown in the
-#'   console.
+#' @param verbose \strong{Deprecated argument. It will be retired in the next release.}
 #' @return
 #'
-#' The following values are returned. Please, note that if a grouping variable
-#' is used, then the results are returned into a list.
+#' If \code{.data} is a grouped data passed from \code{\link[dplyr]{group_by}()}
+#' then the results will be returned into a list-column of data frames.
 #'
 #' * \strong{cormat} A symmetric Pearson's coefficient correlation matrix
 #' between the variables
@@ -41,6 +38,8 @@
 #'
 #' * \strong{det} The determinant of the correlation matrix.
 #'
+#' * \strong{ncorhigh} Number of correlation greather than |0.8|.
+#'
 #' * \strong{largest_corr} The largest correlation (in absolute value) observed.
 #'
 #' * \strong{smallest_corr} The smallest correlation (in absolute value)
@@ -77,19 +76,20 @@
 #'
 #' # Using a data frame
 #' col_diag_gen <- data_ge2 %>%
-#'                 split_factors(GEN) %>%
+#'                 group_by(GEN) %>%
 #'                 colindiag()
 #'
-#' # Diagnostic by levels of a factor selecting desired variables
+#' # Diagnostic by levels of a factor
+#' # For variables with "N" in variable name
 #' col_diag_gen <- data_ge2 %>%
-#'                 split_factors(GEN) %>%
-#'                 colindiag(EH, PH, CD, CL)
+#'                 group_by(GEN) %>%
+#'                 colindiag(contains("N"))
 #'}
 colindiag <- function(.data,
                       ...,
                       by = NULL,
                       n = NULL,
-                      verbose = TRUE) {
+                      verbose = "DEPRECATED") {
   if (!any(class(.data) %in% c("matrix", "data.frame", "split_factors",
                                "covcor_design", "tbl_df"))) {
     stop("The object 'x' must be a correlation matrix, a data.frame or an object of class split_factors")
@@ -100,8 +100,19 @@ colindiag <- function(.data,
   if (is.data.frame(.data) && !is.null(n)) {
     stop("You cannot informe the sample size because a data frame was used as input.")
   }
-  if (is.split_factors(.data) && !is.null(n)) {
-    stop("You cannot informe the sample size because a data frame was used as input.")
+  if (!missing(by)){
+    if(length(as.list(substitute(by))[-1L]) != 0){
+      stop("Only one grouping variable can be used in the argument 'by'.\nUse 'group_by()' to pass '.data' grouped by more than one variable.", call. = FALSE)
+    }
+    .data <- group_by(.data, {{by}})
+  }
+  if(is_grouped_df(.data)){
+    results <- .data %>%
+      doo(colindiag,
+          ...,
+          n = n,
+          verbose = verbose)
+    return(results %>% set_class(c("tbl_df", "tbl",  "data.frame", "colingroup", "colindiag")))
   }
   internal <- function(x) {
     if (is.matrix(x)) {
@@ -136,87 +147,123 @@ colindiag <- function(.data,
                             " = ", round(results[which.min(abs(results$linear)),
                                                  1], 3))
     ncorhigh <- sum(results$linear >= abs(0.8))
-    if (verbose == TRUE) {
-      if (NC > 1000) {
-        cat(paste0("Severe multicollinearity in the matrix! Pay attention on the variables listed bellow\n",
-                   "CN = ", round(NC, 3), "\n"))
-      }
-      if (NC < 100) {
-        cat(paste0("Weak multicollinearity in the matrix\n",
-                   "NC = ", round(NC, 3), "\n"))
-      }
-      if (NC > 100 & NC < 1000) {
-        cat(paste0("The multicollinearity in the matrix should be investigated.\n",
-                   "NC = ", round(NC, 3), "\n", "Largest VIF = ",
-                   max(VIF), "\n"))
-      }
-    }
-    ultimo <- data.frame(Peso = t(AvAvet[c(nrow(AvAvet)),
-                                         ])[-c(1), ])
+    ultimo <- data.frame(Peso = t(AvAvet[c(nrow(AvAvet)), ])[-c(1), ])
     abs <- data.frame(Peso = abs(ultimo[, "Peso"]))
     rownames(abs) <- rownames(ultimo)
-    ultimo <- abs[order(abs[, "Peso"], decreasing = T), ,
-                  drop = FALSE]
+    ultimo <- abs[order(abs[, "Peso"], decreasing = T), , drop = FALSE]
     pesovarname <- paste(rownames(ultimo), collapse = " > ")
-    final <- list(cormat = data.frame(cor.x), corlist = results,
-                  evalevet = AvAvet, VIF = VIF, CN = NC, det = Det,
-                  largest_corr = largest_corr, smallest_corr = smallest_corr,
+    final <- list(cormat = data.frame(cor.x),
+                  corlist = results,
+                  evalevet = AvAvet,
+                  VIF = VIF,
+                  CN = NC,
+                  det = Det,
+                  ncorhigh = ncorhigh,
+                  largest_corr = largest_corr,
+                  smallest_corr = smallest_corr,
                   weight_var = pesovarname)
-    if (verbose == TRUE) {
-      cat(paste0("Matrix determinant: ", round(Det, 7)), "\n")
-      cat(paste0("Largest correlation: ", largest_corr), "\n")
-      cat(paste0("Smallest correlation: ", smallest_corr), "\n")
-      cat(paste0("Number of VIFs > 10: ", length(which(VIF > 10))), "\n")
-      cat(paste0("Number of correlations with r >= |0.8|: ", ncorhigh), "\n")
-      cat(paste0("Variables with largest weight in the last eigenvalues: \n", pesovarname), "\n")
-    }
-    return(invisible(final))
+    invisible(final)
   }
-
   if (is.matrix(.data)) {
     out <- internal(.data)
   }
-  if (!missing(by)){
-    if(length(as.list(substitute(by))[-1L]) != 0){
-      stop("Only one grouping variable can be used in the argument 'by'.\nUse 'split_factors()' to pass '.data' grouped by more than one variable.", call. = FALSE)
-    }
-    .data <- split_factors(.data, {{by}}, verbose = FALSE, keep_factors = TRUE)
-  }
-  if (any(class(.data) %in% c("split_factors", "covcor_design"))) {
-    if(!missing(...)){
-      dfs <- lapply(.data[[1]], function(x){
-        select_cols(x, ...) %>%
-          select_numeric_cols()
-      })
-    } else{
-      dfs <- lapply(.data[[1]], function(x){
-        select_numeric_cols(x)
-      })
-    }
-    out <- lapply(seq_along(dfs), function(x) {
-      if (verbose == TRUE) {
-        cat("\n----------------------------------------------------------------------------\n")
-        cat("Level:", names(dfs)[[x]], "\n")
-        cat("----------------------------------------------------------------------------\n")
-      }
-      internal(dfs[[x]])
-    })
-    names(out) <- names(dfs)
-  }
   if (is.data.frame(.data)) {
     if(!missing(...)){
       dfs <-  select_cols(.data, ...) %>%
         select_numeric_cols()
     } else{
-      if (verbose == TRUE) {
-        if (sum(lapply(.data, is.factor) == TRUE) > 0) {
-          warning("The factors ", paste0(collapse = " ", names(.data[, unlist(lapply(.data, is.factor))])),
-                  " where ignored.  Use 'split_factors()' to perform an analysis for each level of a factor.")
-        }
-      }
       dfs <- select_numeric_cols(.data)
     }
     out <- internal(dfs)
   }
-  invisible(out)
+  invisible(out %>% set_class("colindiag"))
+}
+
+
+#' Print an object of class colindiag
+#'
+#' Print the \code{colindiag} object in two ways. By default, the results
+#' are shown in the R console. The results can also be exported to the directory
+#' into a *.txt file.
+#'
+#'
+#' @param x The object of class \code{colindiag}
+#' @param export A logical argument. If \code{TRUE}, a *.txt file is exported to
+#'   the working directory.
+#' @param file.name The name of the file if \code{export = TRUE}
+#' @param digits The significant digits to be shown.
+#' @param ... Options used by the tibble package to format the output. See
+#'   \code{\link[tibble:formatting]{tibble::print()}} for more details.
+#' @author Tiago Olivoto \email{tiagoolivoto@@gmail.com}
+#' @method print colindiag
+#' @export
+#' @examples
+#' \donttest{
+#' library(metan)
+#' col <- colindiag(data_ge2)
+#' print(col)
+#' }
+print.colindiag <- function(x, export = FALSE, file.name = NULL, digits = 3, ...) {
+  if (export == TRUE) {
+    file.name <- ifelse(is.null(file.name) == TRUE, "colindiag print", file.name)
+    sink(paste0(file.name, ".txt"))
+  }
+  if(any(class(x) == "colingroup")){
+    for (i in 1:nrow(x)){
+      df <- x[i,]
+      names <-
+        select(df, -data) %>%
+        concatenate(everything(), pull = TRUE)
+      cat("Level:", names, "\n")
+      cat("---------------------------------------------------------------------------\n")
+      dat <- df[["data"]][[1]]
+      CN <- dat$CN
+      VIF <- dat$VIF
+    if (CN > 1000) {
+      cat(paste0("Severe multicollinearity in the matrix! Pay attention on the variables listed bellow\n",
+                 "CN = ", round(CN, digits), "\n"))
+    }
+    if (CN < 100) {
+      cat(paste0("Weak multicollinearity in the matrix\n",
+                 "CN = ", round(CN, digits), "\n"))
+    }
+    if (CN > 100 & CN < 1000) {
+      cat(paste0("The multicollinearity in the matrix should be investigated.\n",
+                 "CN = ", round(CN, digits), "\n", "Largest VIF = ",
+                 max(VIF), "\n"))
+    }
+      cat(paste0("Matrix determinant: ", round(dat$det, 7)), "\n")
+      cat(paste0("Largest correlation: ", dat$largest_corr), "\n")
+      cat(paste0("Smallest correlation: ", dat$smallest_corr), "\n")
+      cat(paste0("Number of VIFs > 10: ", length(which(VIF > 10))), "\n")
+      cat(paste0("Number of correlations with r >= |0.8|: ",dat$ncorhigh), "\n")
+      cat(paste0("Variables with largest weight in the last eigenvalues: \n", dat$weight_var), "\n")
+      cat("---------------------------------------------------------------------------\n")
+    }
+  } else{
+    CN <- x$CN
+    VIF <- x$VIF
+    if (CN > 1000) {
+      cat(paste0("Severe multicollinearity in the matrix! Pay attention on the variables listed bellow\n",
+                 "CN = ", round(CN, digits), "\n"))
+    }
+    if (CN < 100) {
+      cat(paste0("Weak multicollinearity in the matrix\n",
+                 "CN = ", round(CN, digits), "\n"))
+    }
+    if (CN > 100 & CN < 1000) {
+      cat(paste0("The multicollinearity in the matrix should be investigated.\n",
+                 "CN = ", round(CN, digits), "\n", "Largest VIF = ",
+                 max(VIF), "\n"))
+    }
+    cat(paste0("Matrix determinant: ", round(x$det, 7)), "\n")
+    cat(paste0("Largest correlation: ", x$largest_corr), "\n")
+    cat(paste0("Smallest correlation: ", x$smallest_corr), "\n")
+    cat(paste0("Number of VIFs > 10: ", length(which(VIF > 10))), "\n")
+    cat(paste0("Number of correlations with r >= |0.8|: ",x$ncorhigh), "\n")
+    cat(paste0("Variables with largest weight in the last eigenvalues: \n", x$weight_var), "\n")
+  }
+  if (export == TRUE) {
+    sink()
+  }
 }
