@@ -30,9 +30,10 @@
 #'
 #' @export
 #' @examples
-#'
+#' \donttest{
 #' library(metan)
 #' out <- gai(data_ge2, ENV, GEN, REP, c(EH, PH, EL, CD, ED, NKE))
+#' }
 #'
 gai <- function(.data, env, gen, rep, resp, verbose = TRUE) {
   factors  <- .data %>%
@@ -40,9 +41,9 @@ gai <- function(.data, env, gen, rep, resp, verbose = TRUE) {
            GEN = {{gen}},
            REP = {{rep}}) %>%
     mutate_all(as.factor)
-  vars <- .data %>%
-    select({{resp}}) %>%
-    select_numeric_cols()
+  vars <- .data %>% select({{resp}}, -names(factors))
+  has_text_in_num(vars)
+  vars %<>% select_numeric_cols()
   listres <- list()
   nvar <- ncol(vars)
   for (var in 1:nvar) {
@@ -50,9 +51,10 @@ gai <- function(.data, env, gen, rep, resp, verbose = TRUE) {
       mutate(Y = vars[[var]])
     temp <- make_mat(data, ENV, GEN, Y) %>%
       gm_mean() %>%
-      as_tibble(rownames = NA) %>%
+      t() %>%
+      as.data.frame() %>%
       rownames_to_column("GEN") %>%
-      mutate(rank = rank(-value)) %>%
+      mutate(rank = rank(-V1)) %>%
       set_names("GEN", "GAI", "GAI_R")
     if (nvar > 1) {
       listres[[paste(names(vars[var]))]] <- temp
