@@ -33,14 +33,14 @@
 #'
 #' @export
 #' @examples
-#'
+#' \donttest{
 #' library(metan)
 #' out <- superiority(data_ge2,
 #'                    env = ENV,
 #'                    gen = GEN,
 #'                    rep = REP,
 #'                    resp = PH)
-#'
+#'}
 #'
 superiority <- function(.data, env, gen, rep, resp, verbose = TRUE) {
   factors  <- .data %>%
@@ -48,9 +48,9 @@ superiority <- function(.data, env, gen, rep, resp, verbose = TRUE) {
            GEN = {{gen}},
            REP = {{rep}}) %>%
     mutate_all(as.factor)
-  vars <- .data %>%
-    select({{resp}}) %>%
-    select_numeric_cols()
+  vars <- .data %>% select({{resp}}, -names(factors))
+  has_text_in_num(vars)
+  vars %<>% select_numeric_cols()
   listres <- list()
   nvar <- ncol(vars)
   for (var in 1:nvar) {
@@ -98,3 +98,56 @@ superiority <- function(.data, env, gen, rep, resp, verbose = TRUE) {
   }
   return(structure(listres, class = "superiority"))
 }
+
+
+
+
+
+
+#' Print an object ofclass \code{superiority}
+#'
+#' Print the \code{superiority} object in two ways. By default, the results are
+#' shown in the R console. The results can also be exported to the directory
+#' into a *.txt file.
+#'
+#'
+#' @param x An object of class \code{superiority}.
+#' @param export A logical argument. If \code{TRUE}, a *.txt file is exported to
+#'   the working directory.
+#' @param file.name The name of the file if \code{export = TRUE}
+#' @param digits The significant digits to be shown.
+#' @param ... Options used by the tibble package to format the output. See
+#'   \code{\link[tibble:formatting]{tibble::print()}} for more details.
+#' @author Tiago Olivoto \email{tiagoolivoto@@gmail.com}
+#' @method print superiority
+#' @export
+#' @examples
+#' \donttest{
+#' library(metan)
+#' model <- superiority(data_ge2, ENV, GEN, REP, PH)
+#' print(model)
+#' }
+print.superiority <- function(x, export = FALSE, file.name = NULL, digits = 3, ...) {
+  if (!class(x) == "superiority") {
+    stop("The object must be of class 'superiority'")
+  }
+  if (export == TRUE) {
+    file.name <- ifelse(is.null(file.name) == TRUE, "superiority summary", file.name)
+    sink(paste0(file.name, ".txt"))
+  }
+  opar <- options(pillar.sigfig = digits)
+  on.exit(options(opar))
+  for (i in 1:length(x)) {
+    var <- x[[i]]
+    cat("Variable", names(x)[i], "\n")
+    cat("---------------------------------------------------------------------------\n")
+    cat("Superiority index considering all, favorable and unfavorable environments\n")
+    cat("---------------------------------------------------------------------------\n")
+    print(var$index)
+    cat("---------------------------------------------------------------------------\n")
+    cat("\n\n\n")
+  }
+  if (export == TRUE) {
+    sink()
+  }
+}
