@@ -51,6 +51,7 @@
 #' Principal Components, the estimates of Weighted Average of Absolute Scores,
 #' and WAASY (the index that consider the weights for stability and productivity
 #' in the genotype ranking.
+#' * \strong{ge_means} A tbl_df containing the genotype-environment means.
 #' * \strong{ge_eff} A \emph{gxe} matrix containing the genotype-environment effects.
 #' * \strong{eigenvalues} The eigenvalues from the singular value decomposition
 #' of the matrix withe the genotype-environment interaction effects.
@@ -77,8 +78,8 @@
 #'                      gen = GEN,
 #'                      resp = everything())
 #' # The index WAAS
-#' get_model_data(model, what = "OrWAASB")
-#' get_model_data(model2, what = "OrWAASB")
+#' get_model_data(model, what = "OrWAAS")
+#' get_model_data(model2, what = "OrWAAS")
 #'
 #'}
 #'
@@ -94,9 +95,9 @@ waas_means <- function(.data,
     select(ENV = {{env}},
            GEN = {{gen}}) %>%
     mutate_all(as.factor)
-  vars <- .data %>%
-    select({{resp}}) %>%
-    select_numeric_cols()
+  vars <- .data %>% select({{resp}}, -names(factors))
+  has_text_in_num(vars)
+  vars %<>% select_numeric_cols()
   nvar <- ncol(vars)
   if (is.null(mresp)) {
     mresp <- replicate(nvar, 100)
@@ -204,6 +205,7 @@ waas_means <- function(.data,
         ungroup()
     }
     temp <- structure(list(model = WAASAbs,
+                           ge_means = data,
                            ge_eff = res_mat,
                            eigenvalues = SS,
                            proportion = weights,
